package com.ehsure.scanner_view;

import android.content.Context;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;

import androidx.annotation.Nullable;

import com.google.zxing.Result;

import java.util.Map;

import cn.bingoogolapple.qrcode.zxing.QRCodeDecoder;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;
import me.dm7.barcodescanner.zxing.ZXingScannerView;

public class ScannerView implements PlatformView, ZXingScannerView.ResultHandler {

    private final ZXingScannerView mScannerView;
    private final View scannerLayout;
    private final MethodChannel methodChannel;

    ScannerView(Context context, BinaryMessenger messenger, int id, Map<String, Object> params) {
        methodChannel = new MethodChannel(messenger, "ehsure_scanner_channel");
        boolean hasFlash = context.getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
        mScannerView = new ZXingScannerView(context);
        methodChannel.setMethodCallHandler((call, result) -> {
            if (call.method.equals("scanResume")) {
                mScannerView.startCamera();
            } else if (call.method.equals("qrImage")) {
                handleQRImage(call, result);
            } else {
                result.notImplemented();
            }
        });
        scannerLayout = LayoutInflater.from(context).inflate(R.layout.scanner_layout, null);
        FrameLayout contentFrame = scannerLayout.findViewById(R.id.content_frame);
        contentFrame.addView(mScannerView);
        ImageView ivFlashlight = scannerLayout.findViewById(R.id.iv_flashlight);
        ivFlashlight.setOnClickListener(v -> {
            if (hasFlash) {
                boolean flash = mScannerView.getFlash();
                if (flash) {
                    ivFlashlight.setImageResource(R.mipmap.ic_flashlight_off);
                } else {
                    ivFlashlight.setImageResource(R.mipmap.ic_flashlight_on);
                }
                mScannerView.setFlash(!flash);
            }
        });
        mScannerView.setSquareViewFinder(true);
        mScannerView.startCamera();
        mScannerView.setResultHandler(this);
    }

    @Override
    public View getView() {
        return scannerLayout;
    }

    @Override
    public void dispose() {
        methodChannel.setMethodCallHandler(null);
        mScannerView.stopCameraPreview();
        mScannerView.stopCamera();
    }

    @Override
    public void handleResult(Result rawResult) {
        methodChannel.invokeMethod("scanResult", rawResult.getText(), new MethodChannel.Result(){
            @Override
            public void success(@Nullable Object result) {
                if (result != null) {
                    mScannerView.resumeCameraPreview(ScannerView.this);
                }
            }

            @Override
            public void error(String errorCode, @Nullable String errorMessage, @Nullable Object errorDetails) {
                Log.e(errorCode + errorMessage, errorDetails.toString());
            }

            @Override
            public void notImplemented() {

            }
        });
    }

    private void handleQRImage(MethodCall call, MethodChannel.Result result) {
        String path = (String)call.arguments;
        new MyTask(path,result).execute();
    }

    static class MyTask extends AsyncTask<Void, Void, String>{

        private final String path;
        private final MethodChannel.Result result;

        public MyTask(String path, MethodChannel.Result result){
            this.path = path;
            this.result = result;
        }

        @Override
        protected String doInBackground(Void... params) {
            return QRCodeDecoder.syncDecodeQRCode(path);
        }

        @Override
        protected void onPostExecute(String s) {
            result.success(s);
        }
    }

}